package com.oracle.ticketsystem.dao;

import java.util.List;

import com.oracle.ticketsystem.beans.Ticket;

/**
 * A DAO that handles the ticket related requests, 
 * for example, adding a new ticket, updating an 
 * existing ticket, getting a specific ticket, and 
 * getting list of tickets by specific state.
 *
 */
public interface ITicketDao {

	/**
	 * Adds the given ticket.
	 * 
	 * @param ticket to add
	 * @return <code>Ticket</code> reference to ticket being added
	 */
	public Ticket add(long productId, 
			String customerName, String customerEmail, 
			String title, String description);
	
	/**
	 * Updates the ticket specified with the given ID.
	 * <p>
	 * Ignores the update request (and returns null), if could not find 
	 * Ticket for the given ID.
	 * <p>
	 * It updates the current state of the ticket and also adds an entry 
	 * for the ticket history.
	 * 
	 * @param ticketId 
	 * 			ID of the ticket to update
	 * @param technicianId
	 * 			ID of technician who is updating the ticket
	 * @param comment
	 * 			string comment about the update
	 * @param state
	 * 			state to update for the ticket
	 * @return the updated <code>TicketType</code> 
	 */
	public Ticket update(long ticketId, String technicianId, 
							 String comment, String state);
	
	/**
	 * Returns <code>Ticket</code> reference for the given ticket ID, 
	 * or null if does not found.
	 * 
	 * @param ticketId
	 * 			ID of ticket to retrieve
	 * @return <code>Ticket</code> reference for the given ticket ID if found, 
	 * otherwise null.
	 */
	public Ticket get(long ticketId);
	
	/**
	 * Returns <code>List</code> of tickets assigned to technician with the given ID.
	 * 
	 * @param technicianId get tickets assigned to technician with the given ID
	 * @return <code>List</code> of tickets.
	 */
	public List<Ticket> getTicketsOwnedByTechnician(String technicianId);
	
	/**
	 * Returns <code>List</code> of open tickets i.e. either NEW or OPEN
	 * 
	 * @return a list of open tickets i.e. either NEW or OPEN
	 */
	public List<Ticket> getOpenTickets();
	
	/**
	 * Removes the ticket with the given ticket ID.
	 * 
	 * @param ticketId 
	 * 			the ID of the ticket to be removed
	 */
	public void remove(long ticketId);
	
}
